<?php

/**
 * DRD Server Operation to find out all domains on thsi Drupal installation.
 *
 * @return type
 */
function drd_server_server_domains() {
  $sites = drd_server_read_sites();
  $output = array();
  $error = array();
  foreach ($sites as $url => $id) {
    if (isset($output[$id])) {
      $output[$id]['aliases'][] = $url;
    }
    else if (isset($error[$id])) {
      // Ignore this alias, it won't work
    }
    else {
      $code = drd_server_server_check_host($url, $id);
      if ($code === FALSE) {
        $error[$id] = TRUE;
      }
      else {
        $output[$id] = array(
          'url' => $url,
          'aliases' => array(),
          'drd installed' => ($code === TRUE),
        );
      }
    }
  }
  return drd_server_result('server.domains', $output);
}

/**
 * Function called by drd_server_server_domains() to check a specific URL
 * if it really exists and if the module drd_server is installed.
 *
 * @param type $url
 * @param type $dir
 * @return boolean
 */
function drd_server_server_check_host($url, $dir) {
  $id = rand();
  try {
    $check = drupal_http_request('http://' . $url . '/admin/drd_server/' . $id);
    $ok = (($check->code == 200) || ($check->code == 302 && $check->redirect_code == 200));
    if ($ok && $check->data == ($id / 2)) {
      return TRUE;
    }
    return $check->code;
  }
  catch (Exception $e) {
    return FALSE;
  }
}

function drd_server_server_list_updates() {
  if (!module_exists('update')) {
    module_enable(array('update'));
  }
  // There doesn't seem to be a way yet to fetch update info on disabled
  // modules as well, so we just live with what we get by default
  $result = drd_server_domain_list_updates('server');

  // Just return result as is, it has been processed already
  return $result;
}

/**
 * DRD Server Operation to retrieve the PHP error log from the OS.
 *
 * @return type
 */
function drd_server_server_php_error_log($max_length = 5000) {
  $log_file = ini_get('error_log');
  if (!file_exists($log_file)) {
    return drd_server_error(t('PHP Error Log does not exist.'), DRD_ERROR_NO_LOG_PHP);
  }
  $offset = max(-1, (filesize($log_file) - $max_length));
  $log = file_get_contents($log_file, FILE_BINARY, NULL, $offset);
  $result['cache']['php_error_log']['server'] = array(
    'title' => 'PHP Error Log',
    'callback' => 'drupal_render',
    'arguments' => array(
      'data' => array(
        '#markup' => '<pre>' . $log . '</pre>',
      ),
    ),
  );
  return drd_server_result('server.php.error.log', $result);
}
